module.exports = function( grunt ) {

  "use strict";

  var gzip = require("gzip-js"),
    readOptionalJSON = function( filepath ) {
      var data = {};
      try {
        data = grunt.file.readJSON( filepath );
      } catch(e) {}
      return data;
    },
    srcHintOptions = readOptionalJSON(".jshintrc");

  // The concatenated file won't pass onevar
  // But our modules can
  delete srcHintOptions.onevar;

  grunt.initConfig({
    pkg: grunt.file.readJSON("package.json"),
    dst: readOptionalJSON("dist/.destination.json"),

    compare_size: {
      files: [ "dist/smart.js", "dist/smart.min.js" ],
      options: {
        compress: {
          gz: function( contents ) {
            return gzip.zip( contents, {} ).length;
          }
        },
        cache: "dist/.sizecache.json"
      }
    },

    concat: {
      options: {
      },
      dist: {
        dest: "dist/smart.js",
        src: [
          "src/intro.js",
          "src/support.js",
          "src/util.js",
          "src/binder.js",
          "src/assets.js",
          "src/view.js",
          "src/box.js",
          "src/background.js",
          "src/border.js",
          //"src/filter.js", TODO
          "src/stage.js",
          "src/text.js",
          "src/canvas.js",
          "src/animation.js",
          "src/timeline.js",
          "src/timetable.js",
          "src/tween.js",
          "src/outro.js"
        ]
      }
    },

    jshint: {
      dist: {
        src: [ "dist/smart.js" ],
        options: srcHintOptions
      },
      grunt: {
        src: [ "Gruntfile.js" ],
        options: {
          jshintrc: ".jshintrc"
        }
      },
      tests: {
        // TODO:
        src: [ "test/data/{test,testinit,testrunner}.js", "test/unit/**/*.js" ],
        options: {
          jshintrc: "test/.jshintrc"
        }
      }
    },

    watch: {
      files: [ "<%= jshint.grunt.src %>", "<%= jshint.tests.src %>", "src/**/*.js" ],
      tasks: "dev"
    },

    uglify: {
      all: {
        files: {
          "dist/smart.min.js": [ "dist/smart.js" ]
        },
        options: {
          // Keep our hard-coded banner
          preserveComments: "some",
          sourceMap: "dist/smart.min.map",
          sourceMappingURL: "smart.min.map",
          report: "min",
          beautify: {
            ascii_only: true
          },
          compress: {
            hoist_funs: false,
            join_vars: false,
            loops: false,
            unused: false
          },
          mangle: {
            // saves some bytes when gzipped
            except: [ "undefined" ]
          }
        }
      }
    }
  });

  // Load grunt tasks from NPM packages
  grunt.loadNpmTasks("grunt-compare-size");
  grunt.loadNpmTasks("grunt-contrib-concat");
  grunt.loadNpmTasks("grunt-contrib-watch");
  grunt.loadNpmTasks("grunt-contrib-jshint");
  grunt.loadNpmTasks("grunt-contrib-uglify");

  // Default grunt
  grunt.registerTask( "default", [ "concat", "jshint", "uglify", "compare_size" ] );

  // Short list as a high frequency watch task
  grunt.registerTask( "dev", [ "concat", "jshint" ] );
};
